@@ -15,13 +15,13 @@
 from __future__ import absolute_import
 import copy_reg
 
-import lru
+import bson.dbref
+import bson.objectid
 import persistent.interfaces
 import persistent.dict
 import persistent.list
 import pymongo.binary
-import pymongo.dbref
-import pymongo.objectid
+import repoze.lru
 import types
 import zope.interface
 from zope.dottedname.resolve import resolve
@@ -32,7 +32,7 @@ IGNORE_IDENTICAL_DOCUMENTS = True
 ALWAYS_READ_FULL_DOC = True
 
 SERIALIZERS = []
-OID_CLASS_LRU = lru.LRUCache(20000)
+OID_CLASS_LRU = repoze.lru.LRUCache(20000)
 COLLECTIONS_WITH_TYPE = set()
 AVAILABLE_NAME_MAPPINGS = set()
 PATH_RESOLVE_CACHE = {}
@@ -43,6 +43,26 @@ def get_dotted_name(obj):
 class PersistentDict(persistent.dict.PersistentDict):
     _p_mongo_sub_object = True
 
+    def __init__(self, data=None, **kwargs):
+        # We optimize the case where data is not a dict. The original
+        # implementation always created an empty dict, which it then
+        # updated. This turned out to be expensive.
+        if data is None:
+            self.data = {}
+        elif isinstance(data, dict):
+            self.data = data.copy()
+        else:
+            self.data = dict(data)
+        if len(kwargs):
+            self.update(kwargs)
+
+    def __getitem__(self, key):
+        # The UserDict supports a __missing__() function, which I have never
+        # seen or used before, but it makes the method significantly
+        # slower. So let's not do that.
+        return self.data[key]
+
+
 class PersistentList(persistent.list.PersistentList):
     _p_mongo_sub_object = True
 
@@ -258,7 +278,7 @@ class ObjectWriter(object):
             obj._p_oid = pymongo.dbref.DBRef(coll_name, doc_id, db_name)
             # Make sure that any other code accessing this object in this
             # session, gets the same instance.
-            self._jar._object_cache[doc_id] = obj
+            self._jar._object_cache[hash(obj._p_oid)] = obj
         else:
             doc['_id'] = obj._p_oid.id
             # We only want to store a new version of the document, if it is
@@ -311,10 +331,9 @@ class ObjectReader(object):
         __traceback_info__ = dbref
         # 1. Check the global oid-based lookup cache. Use the hash of the id,
         #    since otherwise the comparison is way too expensive.
-        try:
-            return OID_CLASS_LRU[hash(dbref.id)]
-        except KeyError:
-            pass
+        klass = OID_CLASS_LRU.get(hash(dbref))
+        if klass is not None:
+            return klass
         # 2. Check the transient single map entry lookup cache.
         try:
             return self._single_map_cache[(dbref.database, dbref.collection)]
@@ -338,7 +357,7 @@ class ObjectReader(object):
                     .find_one(dbref.id, fields=('_py_persistent_type',))
             if '_py_persistent_type' in obj_doc:
                 klass = self.simple_resolve(obj_doc['_py_persistent_type'])
-                OID_CLASS_LRU[hash(dbref.id)] = klass
+                OID_CLASS_LRU.put(hash(dbref), klass)
                 return klass
         # 4. Try to resolve the path directly. We want to do this optimization
         #    after all others, because trying it a lot is very expensive.
@@ -403,7 +422,7 @@ class ObjectReader(object):
                         break
                 else:
                     raise ImportError(dbref)
-            OID_CLASS_LRU[dbref.id] = klass
+            OID_CLASS_LRU.put(hash(dbref), klass)
             return klass
 
     def get_non_persistent_object(self, state, obj):
@@ -432,14 +451,14 @@ class ObjectReader(object):
         return sub_obj
 
     def get_object(self, state, obj):
-        if isinstance(state, pymongo.objectid.ObjectId):
+        if isinstance(state, bson.objectid.ObjectId):
             # The object id is special. Preserve it.
             return state
         if isinstance(state, pymongo.binary.Binary):
             # Binary data in Python 2 is presented as a string. We will
             # convert back to binary when serializing again.
             return str(state)
-        if isinstance(state, pymongo.dbref.DBRef):
+        if isinstance(state, bson.dbref.DBRef):
             # Load a persistent object. Using the get_ghost() method, so that
             # caching is properly applied.
             return self.get_ghost(state)
@@ -513,13 +532,14 @@ class ObjectReader(object):
         # Set the state.
         obj.__setstate__(state)
 
-    def get_ghost(self, dbref):
+    def get_ghost(self, dbref, klass=None):
         # If we can, we return the object from cache.
         try:
-            return self._jar._object_cache[dbref.id]
+            return self._jar._object_cache[hash(dbref)]
         except KeyError:
             pass
-        klass = self.resolve(dbref)
+        if klass is None:
+            klass = self.resolve(dbref)
         obj = klass.__new__(klass)
         obj._p_jar = self._jar
         obj._p_oid = dbref
@@ -530,5 +550,5 @@ class ObjectReader(object):
         obj._p_mongo_collection = dbref.collection
         # Adding the object to the cache is very important, so that we get the
         # same object reference throughout the transaction.
-        self._jar._object_cache[dbref.id] = obj
+        self._jar._object_cache[hash(dbref)] = obj
         return obj
